@@ -5,14 +5,15 @@
 # SL.gam.3 <- function(...,deg.gam = 3) SL.gam(..., deg.gam = deg.gam)
 
 SL.gam <- function(Y, X, newX, family, obsWeights, deg.gam = 2, cts.num = 4, ...) {
-  .SL.require('gam')
+	# using require instead of requireNamespace() to allow the formula to parse correctly with s(), gam::s() doesn't work, is not recognized as a special function
+  if(!require('gam')) {stop("SL.gam requires the gam package, but it isn't available")} 
   if("mgcv" %in% loadedNamespaces()) warning("mgcv and gam packages are both in use. You might see an error because both packages use the same function names.")
   # create the formula for gam with a spline for each continuous variable
   cts.x <- apply(X, 2, function(x) (length(unique(x)) > cts.num))
   if (sum(!cts.x) > 0) { 
-    gam.model <- as.formula(paste("Y~", paste(paste("gam::s(", colnames(X[, cts.x, drop = FALSE]), ",", deg.gam,")", sep=""), collapse = "+"), "+", paste(colnames(X[, !cts.x, drop=FALSE]), collapse = "+")))
+    gam.model <- as.formula(paste("Y~", paste(paste("s(", colnames(X[, cts.x, drop = FALSE]), ",", deg.gam,")", sep=""), collapse = "+"), "+", paste(colnames(X[, !cts.x, drop=FALSE]), collapse = "+")))
   } else {
-    gam.model <- as.formula(paste("Y~", paste(paste("gam::s(", colnames(X[, cts.x, drop = FALSE]), ",", deg.gam, ")", sep=""), collapse = "+")))
+    gam.model <- as.formula(paste("Y~", paste(paste("s(", colnames(X[, cts.x, drop = FALSE]), ",", deg.gam, ")", sep=""), collapse = "+")))
   }
   # fix for when all variables are binomial
   if (sum(!cts.x) == length(cts.x)) {
